package de.pincservices.pdf;

import org.apache.commons.lang.builder.EqualsBuilder;

import java.util.HashMap;
import java.util.Map;

/**
 * Holds properties and returns {@link Value} objects that contain convenience methods.
 */
public class Configuration {

    private static final Value NULL_VALUE = new Value(null);
    private Map<String, Value> _properties;

    public Configuration() {
        _properties = new HashMap<String, Value>();
    }

    public static Configuration forMap(Map<String, Object> properties) {
        Configuration configuration = new Configuration();
        if (properties != null) {
            for (Map.Entry<String, Object> entry : properties.entrySet()) {
                configuration.setProperty(entry.getKey(), entry.getValue());
            }
        }
        return configuration;
    }

    public void setProperty(String name, Object value) {
        _properties.put(name, new Value(value));
    }

    public Value getProperty(String name) {
        return contains(name) ? _properties.get(name) : NULL_VALUE;
    }

    public boolean contains(String name) {
        return _properties.containsKey(name);
    }

    public Map<String, Object> asMap() {
        final HashMap<String, Object> map = new HashMap<String, Object>();
        for (Map.Entry<String, Value> entry : _properties.entrySet()) {
            map.put(entry.getKey(), entry.getValue()._rawValue);
        }
        return map;
    }

    public static class Value {

        private Object _rawValue;

        public Value(Object rawValue) {
            _rawValue = rawValue;
        }

        public String asString() {
            return _rawValue != null ? _rawValue.toString() : null;
        }

        public boolean is(Object o) {
            return equals(o);
        }

        public boolean equals(Object o) {
            return EqualsBuilder.reflectionEquals(_rawValue, o);
        }

        public String toString() {
            return asString();
        }
    }
}
