package de.pincservices.pdf;


import com.itextpdf.text.pdf.PdfImportedPage
import com.itextpdf.text.pdf.PdfReader
import com.itextpdf.text.pdf.PdfWriter

public class DocumentLayer {

    private PdfReader _reader;
    private int _currentPage;
    private boolean _repeating;
    private boolean _finished;

    public DocumentLayer(String filename, boolean repeating) {
        this(new FileInputStream(filename), repeating);
    }

    public DocumentLayer(InputStream inputStream, boolean repeating) {
        this(repeating);
        _reader = new PdfReader(inputStream);
    }

    public DocumentLayer(boolean repeating) {
        _currentPage = 1;
        _repeating = repeating;
        _finished = false;
    }

    public int getCurrentPage() {
        return _currentPage;
    }

    public void incCurrentPage() {
        _currentPage++;
        _finished = getCurrentPage() > getNumberOfPages();
    }

    public void resetCurrentPage() {
        _currentPage = 1;
        _finished = !isRepeating();
    }

    public int getNumberOfPages() {
        return _reader.getNumberOfPages();
    }

    public boolean isRepeating() {
        return _repeating;
    }

    public boolean isFinished() {
        return _finished;
    }

    public void writeCurrentPage(PdfWriter writer) {

        if (getCurrentPage() > getNumberOfPages()) {
            resetCurrentPage();
        }

        if (!isFinished()) {
            writePage(writer, getCurrentPage());
        }

        incCurrentPage();
    }

    private void writePage(PdfWriter writer, int pageNumber) {
        PdfImportedPage importedPage = writer.getImportedPage(_reader, pageNumber);
        writer.getDirectContent().addTemplate(importedPage, 0, 0);
    }
}
