package de.pincservices.pdf.preprocessor;


import de.pincservices.pdf.Configuration
import de.pincservices.pdf.RenderingPreProcessor
import de.pincservices.template.TemplateEngine
import org.slf4j.Logger
import org.slf4j.LoggerFactory

/**
 * Abstract base class for all template evaluators that implements the common processing methods.
 */
public abstract class AbstractTemplateEvaluator implements RenderingPreProcessor {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractTemplateEvaluator.class);

    private final TemplateEngine _templateEngine;
    private Configuration _configuration;

    public AbstractTemplateEvaluator(Configuration configuration) {
        _configuration = configuration
        _templateEngine = initTemplateEngine()
    }

    protected abstract TemplateEngine initTemplateEngine();

    @Override
    public List<String> processContent(List<String> contents) {
        // Required because the collect closure is executed in the scope of the velocity evaluator that has no knowledge of
        // these private properties. Similar problem already reported (GROOVY-3142).
        def templateEngine = _templateEngine
        def parameters = _configuration.asMap()
        def result = contents?.collect { templateEngine.replacePlaceholdersInText(it, parameters) }
        return result == null ? [] : result
    }

    @Override
    public List<String> processFile(String filename) {
        if (filename.isEmpty()) {
            throw new IllegalArgumentException("Could not process file. Filename must not be null")
        }
        def result = []
        try {
            def fileContent = (new File(filename)).getText()
            result = processContent(Collections.singletonList(fileContent))
        } catch (IOException e) {
            LOGGER.error("Could not process file '" + filename + "'.", e)
        }
        return result;
    }

    protected Configuration getConfiguration() {
        return _configuration;
    }
}
