package de.pincservices.pdf.preprocessor;

import de.pincservices.pdf.Configuration;
import de.pincservices.template.TemplateDirective;
import de.pincservices.template.TemplateEngine;
import de.pincservices.template.VelocityTemplateEngine;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class VelocityEvaluator extends AbstractTemplateEvaluator {

    public VelocityEvaluator(Configuration configuration) {
        super(configuration);
    }

    /**
     * May throw a {@link RuntimeException} if anything unexpected happens.
     */
    @Override
    protected TemplateEngine initTemplateEngine() {
        final VelocityTemplateEngine velocityTemplateEngine = new VelocityTemplateEngine();
        try {
            velocityTemplateEngine.initConfiguration(getDirectives());
        } catch (Exception e) {
            throw new RuntimeException("Template engine configuration could not be initialized.", e);
        }
        return velocityTemplateEngine;
    }

    private List<TemplateDirective> getDirectives() {
        final List<TemplateDirective> result = new ArrayList<TemplateDirective>();
        final Configuration configuration = getConfiguration();
        if (configuration != null) {
            final Map<String,Object> map = configuration.asMap();
            for (Map.Entry<String, Object> entry : map.entrySet()) {
                if (TemplateDirective.class.isAssignableFrom(entry.getValue().getClass())) {
                    result.add((TemplateDirective) entry.getValue());
                }
            }
        }
        return result;
    }
}
