package de.pincservices.pdf.renderer;

import com.lowagie.text.DocumentException;
import org.w3c.dom.Document;
import org.xhtmlrenderer.layout.SharedContext;
import org.xhtmlrenderer.pdf.ITextRenderer;
import org.xhtmlrenderer.pdf.ITextUserAgent;
import org.xhtmlrenderer.resource.XMLResource;
import org.xml.sax.InputSource;

import java.io.BufferedReader;
import java.io.OutputStream;
import java.io.StringReader;
import java.util.Iterator;
import java.util.List;

public class FlyingSaucerPdfRenderer extends AbstractRenderer {

    private ITextRenderer _renderer = new ITextRenderer();

    @Override
    public void createFromContent(List<String> pages, OutputStream outputStream, String externalResourcesBaseUrl) {
        if (pages.size() > 0) {
            initUserAgentCallback();
            final Iterator<String> iterator = pages.iterator();
            try {
                final Content firstPageContent = new Content(iterator.next(), externalResourcesBaseUrl);
                renderFirstPage(outputStream, firstPageContent);
                while (iterator.hasNext()) {
                    final Content content = new Content(iterator.next(), externalResourcesBaseUrl);
                    renderNextPage(content);
                }
            } catch (DocumentException e) {
                throw new RuntimeException("Could not completely create pdf content for " + pages + ".", e);
            } finally {
                _renderer.finishPDF();
            }
        }
    }

    private void renderNextPage(Content content) throws DocumentException {
        setPageContent(content);
        _renderer.writeNextDocument();
    }

    private void renderFirstPage(OutputStream outputStream, Content content) throws DocumentException {
        setPageContent(content);
        _renderer.createPDF(outputStream, false);
    }

    private void setPageContent(Content content) {
        InputSource is = new InputSource(new BufferedReader(new StringReader(content.getRawContent())));
        Document dom = XMLResource.load(is).getDocument();
        _renderer.setDocument(dom, content.getReferenceBaseUrl());
        _renderer.layout();
    }

    private void initUserAgentCallback() {
        final SharedContext sharedContext = _renderer.getSharedContext();
        final ITextUserAgent callback = new ITextUserAgent(_renderer.getOutputDevice());
        callback.setSharedContext(sharedContext);
        sharedContext.setUserAgentCallback(callback);
    }

    private class Content {
        private String _rawContent;
        private String _referenceBaseUrl;

        private Content(String rawContent, String referenceBaseUrl) {
            _rawContent = rawContent;
            _referenceBaseUrl = referenceBaseUrl;
        }

        public String getRawContent() {
            return _rawContent;
        }

        public String getReferenceBaseUrl() {
            return _referenceBaseUrl;
        }
    }
}
