package de.pincservices.template;

import freemarker.ext.beans.BeansWrapper;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateModelException;
import freemarker.template.TemplateScalarModel;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.List;
import java.util.Map;

public class FreeMarkerTemplateEngine extends AbstractTemplateEngine {

	private Configuration _cfg = new Configuration();

    public FreeMarkerTemplateEngine() {
        this(null);
    }

    public FreeMarkerTemplateEngine(String templateLocation) {
        super(templateLocation);
    }

    @Override
	public void initConfiguration(List<TemplateDirective> templateDirectives) throws IOException {
		// Specify the data source where the template files come from.
		// Here I set a file directory for it:
		_cfg.setDirectoryForTemplateLoading(new File(getTemplateLocation()));
		
		// Specify how templates will see the data model:
		BeansWrapper beanWrapper = BeansWrapper.getDefaultInstance();
		beanWrapper.setNullModel(TemplateScalarModel.EMPTY_STRING);
		beanWrapper.setStrict(false);
		_cfg.setObjectWrapper(beanWrapper);

        if (templateDirectives != null) {
            for (TemplateDirective templateDirective : templateDirectives) {
                try {
                    _cfg.setSharedVariable(templateDirective.getName(), templateDirective);
                } catch (TemplateModelException e) {
                    e.printStackTrace();
                }
            }
        }
	}
	
	/**
	 * Replace all placeholders in given raw string by FreeMarker. Null values or non-existing attributes are replaced by an empty string.
	 * @param raw raw content template for FreeMarker
	 * @param parameters data model for FreeMarker
	 * @return Text with replaced placeholders.
	 * @throws Exception
	 */
	@Override
	public String replacePlaceholdersInText(String raw, Map<String, Object> parameters) throws Exception {
		StringReader reader = new StringReader(raw);
		Template temp = new Template(getTemplateName(), reader, _cfg);
		return replacePlaceholders(temp, parameters);
	}
	
	@Override
	public String replacePlaceholdersInFile(String templateFile, Map<String, Object> parameters) throws Exception {
		return replacePlaceholders(_cfg.getTemplate(templateFile), parameters);
	}

	private String replacePlaceholders(Template template, Map<String, Object> parameters) throws Exception {
		final StringWriter out = new StringWriter();
		template.process(parameters, out);
		out.flush();  
		return out.getBuffer().toString();
	}

}
