package de.pincservices.template;

import org.apache.commons.collections.ExtendedProperties;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.runtime.RuntimeSingleton;

import java.io.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;


public class VelocityTemplateEngine extends AbstractTemplateEngine {

    private static final String USER_DIRECTIVE_KEY = "userdirective";

    public VelocityTemplateEngine() {
        this(null);
    }

    public VelocityTemplateEngine(String templateLocation) {
        super(templateLocation);
    }

    @Override
    public void initConfiguration(List<TemplateDirective> templateDirectives) throws Exception {
        final Collection<String> userDirectives = new ArrayList<String>();
        if (templateDirectives != null) {
            for (TemplateDirective templateDirective : templateDirectives) {
                final String className = templateDirective.getVariableClass().getName();
                userDirectives.add(className);
            }
        }

        if (!userDirectives.isEmpty()) {
            setUserDirectives(userDirectives);

        }

        Velocity.init();
    }

    private void setUserDirectives(Collection<String> userDirectives) {
        if (RuntimeSingleton.isInitialized()) {
            for (String userDirective : userDirectives) {
                RuntimeSingleton.loadDirective(userDirective);
            }
        } else {
            // configuration must be injected before Velocity.init() call ...
            ExtendedProperties configuration = new ExtendedProperties();
            configuration.addProperty(USER_DIRECTIVE_KEY, userDirectives);
            Velocity.setExtendedProperties(configuration);
        }
    }

    @Override
    public String replacePlaceholdersInText(String raw, Map<String, Object> parameters) throws Exception {
        final StringReader reader = new StringReader(raw);
        return replacePlaceholders(reader, parameters);
    }

    @Override
    public String replacePlaceholdersInFile(String templateFile, Map<String, Object> parameters) throws Exception {
        FileReader reader = new FileReader(getTemplateLocation() + File.separator + templateFile);
        return replacePlaceholders(reader, parameters);
    }

    private String replacePlaceholders(Reader reader, Map<String, Object> parameters) throws Exception {

        final VelocityContext ctx = new VelocityContext();

        if (parameters != null) {
            for (Map.Entry<String, Object> parameter : parameters.entrySet()) {
                ctx.put(parameter.getKey(), parameter.getValue());
            }
        }

        final StringWriter writer = new StringWriter();

        Velocity.evaluate(ctx, writer, getTemplateName(), reader);

        return writer.getBuffer().toString();
    }
}
