package de.pincservices.template;

import de.pincservices.template.freemarker.AbstractFreeMarkerTemplateDirective;
import freemarker.core.Environment;
import freemarker.template.TemplateDirectiveBody;
import freemarker.template.TemplateException;
import freemarker.template.TemplateModel;
import freemarker.template.TemplateModelException;

import java.io.IOException;
import java.io.Writer;
import java.util.Map;

public class EmailPrefixFreeMarkerTemplateDirective extends AbstractFreeMarkerTemplateDirective {

    public EmailPrefixFreeMarkerTemplateDirective() {
        super("emailPrefix");
    }

    @Override
    public Class getVariableClass() {
        return this.getClass();
    }

    @Override
    public void execute(Environment environment, Map params, TemplateModel[] templateModels, TemplateDirectiveBody body) throws TemplateException, IOException {
        if (!params.isEmpty()) {
            throw new TemplateModelException(
                    "This directive doesn't allow parameters.");
        }
        if (templateModels.length != 0) {
                throw new TemplateModelException(
                    "This directive doesn't allow loop variables.");
        }

        // If there is non-empty nested content:
        if (body != null) {
            body.render(new EmailPrefixWriter(environment.getOut()));
        } else {
           // throw new RuntimeException("missing body");
            final EmailPrefixWriter writer = new EmailPrefixWriter(environment.getOut());
            writer.write(new char[0], 0, 0);
        }


    }

	private class EmailPrefixWriter extends Writer {

        private Writer out;

        EmailPrefixWriter(Writer out) {
            this.out = out;
        }

        @Override
        public void write(char[] cbuf, int off, int len) throws IOException {
        	out.write("email: " + new String(cbuf, off, len));
        }

        @Override
        public void flush() throws IOException {
            out.flush();
        }

        @Override
        public void close() throws IOException {
            out.close();
        }
    }
}
