package de.pincservices.template;

import org.junit.Test;

import java.util.Collections;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

public class FreemarkerTemplateEngineTest {

    private static final String templateLocation = "src/test/resources/templates";

    @Test
    public void testInitConfigurationWithoutFunctions() throws Exception {
        FreeMarkerTemplateEngine engine = new FreeMarkerTemplateEngine(templateLocation);
        // Assume that no exception will be thrown.
        engine.initConfiguration(null);
    }

    @Test
    public void testConfigurationWithFunctions() throws Exception {
        FreeMarkerTemplateEngine engine = new FreeMarkerTemplateEngine(templateLocation);
        TemplateDirective function = new EmailPrefixFreeMarkerTemplateDirective();
        // Assume that no exception will be thrown.
        engine.initConfiguration(Collections.singletonList(function));
    }

    @Test
    public void testReplacement() throws Exception {
        FreeMarkerTemplateEngine engine = new FreeMarkerTemplateEngine(templateLocation);
        TemplateDirective function = new EmailPrefixFreeMarkerTemplateDirective();
        // Assume that no exception will be thrown.
        engine.initConfiguration(Collections.singletonList(function));
        final String result1 = engine.replacePlaceholdersInText("my <@emailPrefix>porscha@pincservices.de</@emailPrefix>", null);
        assertThat(result1, is("my email: porscha@pincservices.de"));
        final String result2 = engine.replacePlaceholdersInText("my <@emailPrefix />porscha@pincservices.de", null);
        assertThat(result2, is("my email: porscha@pincservices.de"));
    }
}
